//
//	Copyright (c) 1995 Bjorn Backlund.  
//	All rights reserved. 
//

#import "app.h"


@implementation Cosmo_View (Pasteboard)

//
// Copy
//

- copy:sender;
{
	const char *types[4];
	int num = 0;
	id pasteboard;
	
    // This message is sent when a user select the "Copy" 
    // option in the Cosmo menu
	
    // Determine if there is something to copy
    
    	if(theSelectedPolyline == nil) {
		NXBeep();
		return self;
	}
	
    // 	Get the global pasteboard
    
    	pasteboard = [Pasteboard new];
	
    // Tell pasteboard what we will put on it
    
    // We will put Cosmo internal data on the pasteboard
    
    	types[num++] = TheCosmoPboardType;
	
    // We will put EPS data on the pasteboard
    
    	types[num++] = NXPostScriptPboardType;
	
    // OOE Level 2: we can also put the Cosmo OOE data

    	types[num++] = TheCosmoOOEPboardType;
	
    // OOE Level 2: we can also put OOE source application information

    	types[num++] = TheOOE_ApplicationInfoPboardType;
	
    // Declare
    
    	[pasteboard declareTypes: types
		num: num
		owner:self];
    
    // We actually don't put any data on the pasteboard 
    // until somebody requests it
    
    // When data actually is need on the pasteboard, 
    // the pasteboard will
    // send the message pasteboard:provideData:type to the
    // view (= self)
    
    	return self;
}

- pasteboard:sender provideData:(NXAtom)type;
{
    // Sent from the pasteboard when data needs to be written
    
    // Note: in this simple implementation we don't take care
    // of the case when the user changes the selection before
    // pasting. Neither do we take care of the case where 
    // the user closes the document before pasting
    
    // Is internal data requested?
    
    	if(type == TheCosmoPboardType)
		[self writePolyline: theSelectedPolyline 
			toPasteboard: sender];
	
    // Is PostScript requested?
    
    	if(type == NXPostScriptPboardType)
		[self writePolyline: theSelectedPolyline 
			asPostScriptToPasteboard: sender];
	
    // OOE Level 2: Is OOE source application info requested?
    
    	if(type == TheOOE_ApplicationInfoPboardType)
		[self writeOOESourceApplicationToPasteboard: sender];
	
    // OOE Level 2: Is OOE data requested?
    
    	if(type == TheCosmoOOEPboardType)
		[self writeOOEDataToPasteboard: sender];
	
	return self;
}

- writePolyline:aPolyline toPasteboard: aPasteboard;
{
	char *buffer;
	int length;
	
    // Write data for the polyline to the pasteboard
    
    // Write the polyline object to a temporary buffer
    
    	buffer = NXWriteRootObjectToBuffer(aPolyline, &length);
	
    // Copy this buffer to the pasteboard
    
	[aPasteboard writeType: TheCosmoPboardType
		data: buffer
		length: length];
		
    // Free the temporary buffer
    
    	NXFreeObjectBuffer(buffer, length);
    	
	return self;
}

- writeOOESourceApplicationToPasteboard:aPasteboard;
{
	const char *name = "Cosmo";
	
    // The source application is of course Cosmo, so we just
    // write this string to the pasteboard
	
	[aPasteboard writeType: TheOOE_ApplicationInfoPboardType
		data: name
		length: strlen(name)];
	
	return self;
}

- writeOOEDataToPasteboard:aPasteboard;
{
	id currentPolylines;
	id tempList;
	
    // Write OOE data for the selection to the pasteboard
    	
    // This is the tricky method. We have selected
    // one of the polylines, and now
    // we want to copy data to the pasteboard 
    // for a whole Cosmo document
    
    // The trick is to temporarily set the list 
    // of polylines of the document
    // to a list of just the selected polyline 
    // and use the methods we
    // already have written for copying a 
    // Cosmo document to the OOE
    // pasteboard
    
    // Get the current list fo polylines
    
    	currentPolylines = [theDocument thePolylines];
	
    // Create a temporary list
    
    	tempList = [[List allocFromZone:[self zone]] init];
	
    // Add the selected polyline to this list
    
    	[tempList addObject: theSelectedPolyline];
	
    // Use the writeDataToPasteboard: method we already have
    // written for OOE Level 1
	
	[theDocument writeDataToPasteboard:aPasteboard]; 
	
    // Cleanup
    
    // Reset the polyline list
    
    	[theDocument setThePolylines: currentPolylines];
	
    // Free the temp list
    
    	[tempList free];
    
    	return self; 
}

//
// Paste
//

- paste:sender;
{
	id pasteboard;
	id aPolyline;
	id polylines;
	const char *ftype;
	char *buffer;
	int length;
	
    // Get the global pasteboard object
    
    	pasteboard = [Pasteboard new];
	
    // Determine if there is anything we can paste
    
    	ftype = [pasteboard findAvailableTypeFrom: 
					&TheCosmoPboardType
				num:1];
				
	if(ftype == NULL) {
		
		NXBeep();
		return self;
	}
	
    // Read data buffer from pasteboard
    
    	[pasteboard readType: TheCosmoPboardType
			data:&buffer 
			length:&length];
			
    // Read object from pasteboard buffer
    
    	aPolyline = NXReadObjectFromBuffer(buffer, length);
	
    // Deallocate data read from pasteboard
    
    	[pasteboard deallocatePasteboardData:buffer length:length];

    // Add polyline to the document
    
    	polylines = [theDocument thePolylines];
	[polylines addObject: aPolyline];
	
    // Set the dirty bit
    
    	[window setDocEdited:1];

   // Redisplay
    
    	[self display];
	
	return self;
}

//
// EPS
//

- writePolyline:aPolyline asPostScriptToPasteboard: aPasteboard;
{
	NXStream *stream;
	
    // Write EPS for the polyline to the pasteboard
    
    // Create a temporary stream to be filled with EPS data
    
    	stream = NXOpenMemory(NULL, 0, NX_WRITEONLY);
	
    // Produce EPS into this stream
	
	[self writePolyline: aPolyline asPostScriptToStream:stream];
	
    // Copy data from the stream to the pasteboard
    
	[aPasteboard writeType: NXPostScriptPboardType
		fromStream: stream];
		
    // Free the temporary stream
    
    	NXCloseMemory(stream, NX_FREEBUFFER);
    	
	return self;
}

- writePolyline: aPolyline asPostScriptToStream: (NXStream *)aStream;
{
	id tempList;
	id oldPolylines;
	NXRect aRect;
	
    // The trick here is to temporary change the list of
    // polylines hold by the document to a list of the polyline
    // we want to produce EPS for
	
    // Create a temporary list of polylines
    
	tempList = [[List allocFromZone:[self zone]] init];
	
    // Add the polyline in question to the temporary list
    
    	[tempList addObject: aPolyline];
	
    // Let the document use this list for a while
    
    	oldPolylines = [theDocument thePolylines];
	[theDocument setThePolylines: tempList];
	
    // Calculate bounding box for the poolyline
    
    	[aPolyline calculateBoundingBox: &aRect];
	
    // Add margins to this rect
    
    	NXInsetRect(&aRect, -10, -10);
	
    // Use the inherited method copyPSCodeInside:to: to 
    // produce EPS for the polyline
    
    	[self copyPSCodeInside: &aRect to: aStream];
	
    // Cleanup: restore the old polyline list and free the
    // temporary one
    
    	[theDocument setThePolylines: oldPolylines];
    	[tempList free];
	
	return self;
}




@end


