#import "draw.h"

extern NXAtom NXDataLinkPboardType;

/* This file is best read in a window as wide as this comment (so that this comment fits on one line). */


// OOE 2.0 Moved these methods to category files
//
// @interface GraphicView(PrivateMethods)
//
// Private methods/
//
// - selectionCache;
// - getBBox:(NXRect *)bbox of:(List *)list extended:(BOOL)extended;
// - recacheSelection:(BOOL)updateLinks;
// - compositeSelection:(const NXRect *)sbounds from:(int)gstate;
// - (int)cacheSelection;
// - dirty:sender;
//
// - resetGUP;
// - (BOOL)move:(NXEvent *)event;
// - moveGraphicsBy:(NXPoint *)vector andDraw:(BOOL)drawFlag;
// - dragSelect:(NXEvent *)event;
// - alignGraphicsBy:(AlignmentType)alignType edge:(NXCoord *)edge; 
// - alignBy:(AlignmentType)alignType; 
// @end




const char *DrawPboardType = "Draw Graphic List Type version 3.0";

BOOL InMsgPrint = NO;		/* whether we are in msgPrint: */


@implementation GraphicView : View
/*
 * The GraphicView class is the core of a DrawDocument.
 *
 * It overrides the View methods related to drawing and event handling
 * and allows manipulation of Graphic objects.
 *
 * The user is allowed to select objects, move them around, group and
 * ungroup them, change their font, and cut and paste them to the pasteboard.
 * It understands multiple formats including PostScript and TIFF as well as
 * its own internal format.  The GraphicView can also import PostScript and
 * TIFF documents and manipulate them as Graphic objects.
 *
 * This is a very skeleton implementation and is intended purely for
 * example purposes.  It should be very easy to add new Graphic objects
 * merely by subclassing the Graphic class.  No code need be added to the
 * GraphicView class when a new Graphic subclass is added.
 *
 * Moving is accomplished using a selection cache which is shared among
 * all instances of GraphicView in the application.  The objects in the
 * selection are drawn using opaque ink on a transparent background so
 * that when they are moved around, the user can see through them to the
 * objects that are not being moved.
 *
 * All of the drawing is done in an off-screen window which is merely
 * composited back to the screen.  This makes for very fast redraw of
 * areas obscured either by the selection moving or the user's scrolling.
 *
 * The glist instance variable is just an ordered list of all the Graphics
 * in the GraphicView.  The slist is an ordered list of the Graphic objects
 * in the selection.  In the original Draw code it was almost always kept 
 * in the same order as the glist, but could be jumbled by doing a shift-drag 
 * select to add to an existing selection. We are now extremely careful about 
 * keeping the slist in the same order as the glist.
 *
 * cacheWindow is the off-screen window into which the objects are
 * drawn.  Flags:  grid is the distance between pixels in the grid
 * imposed on drawing; cacheing is used so that drawSelf:: knows when
 * to composite from the off-screen cache and when to draw into the
 * off-screen cache; groupInSlist is used to keep track of whether a
 * Group Graphic is in the slist so that it knows when to highlight
 * the Ungroup entry in the menu.
 *
 * This class should be able to be used outside the context of this
 * application since it takes great pains not to depend on any other objects
 * in the application.
 */

/*
 * Of course, one should NEVER use global variables in an application, but
 * the following is necessary.  DrawStatus is
 * analogous to the Application Kit's NXDrawingStatus and reflects whether
 * we are in some modal loop.  By definition, that modal loop is "atomic"
 * since we own the mouse during its duration (of course, all bets are off
 * if we have multiple mice!).
 */
 
DrawStatusType DrawStatus = Normal;  /* global state reflecting what we
					are currently doing (resizing, etc.) */

const float DEFAULT_GRID_GRAY = 0.8333;

static Graphic *currentGraphic = nil;	/* won't be used if NXApp knows how
					   to keep track of the currentGraphic */

float KeyMotionDeltaDefault = 0.0;	// OOE 2.0: removed static decl

/* Private C functions needed to implement methods in this class. */

static Window *createCache(NXSize *size, NXZone *zone)
/*
 * Creates an appropriately size off-screen window to cache bits in.
 */
{
    NXRect cacheRect;

    cacheRect.origin.x = 0.0;
    cacheRect.origin.y = 0.0;
    cacheRect.size = *size;

    return [[[Window allocFromZone:zone] initContent:&cacheRect
					       style:NX_PLAINSTYLE
					     backing:NX_RETAINED
					  buttonMask:0
					       defer:NO] reenableDisplay];
}


void getRegion(NXRect *region, const NXPoint *p1, const NXPoint *p2)	// OOE 2.0: removed static decl
/*
 * Returns the rectangle which has p1 and p2 as its corners.
 */
{
    region->size.width = p1->x - p2->x;
    region->size.height = p1->y - p2->y;
    if (region->size.width < 0.0) {
	region->origin.x = p2->x + region->size.width;
	region->size.width = ABS(region->size.width);
    } else {
	region->origin.x = p2->x;
    }
    if (region->size.height < 0.0) {
	region->origin.y = p2->y + region->size.height;
	region->size.height = ABS(region->size.height);
    } else {
	region->origin.y = p2->y;
    }
}



 
BOOL checkForGroup(List *list)				// OOE 2.0: removed static decl
/*
 * Looks through the given list searching for objects of the Group class.
 * We use this to keep the gvFlags.groupInSlist flag up to date when things
 * are removed from the slist (the list of selected objects).  That way
 * we can efficiently keep the Ungroup menu item up to date.
 */
{
    int i = [list count];
    while (i--) if ([[list objectAt:i] isKindOf:[Group class]]) return YES;
    return NO;
}




/* Hack to support growable Text objects. */

static View *createEditView(GraphicView *self)
/*
 * editView is essentially a dumb, FLIPPED (with extra emphasis on the
 * flipped) subview of our GraphicView which completely covers it and
 * which automatically sizes itself to always completely cover the
 * GraphicView.  It is necessary since growable Text objects only work
 * when they are subviews of a flipped view.
 *
 * See TextGraphic for more details about why we need editView
 * (it is purely a workaround for a limitation of the Text object).
 */
{
    View *view;

    [self setAutoresizeSubviews:YES];
    view = [[View allocFromZone:[self zone]] initFrame:&(self->frame)];
    [view setFlipped:YES];
    [view setAutosizing:NX_WIDTHSIZABLE|NX_HEIGHTSIZABLE];
    [self addSubview:view];

    return view;
}

/* Factory methods. */

+ initialize
/*
 * We up the version of the class so that we can read old .draw files.
 * See the read: method for how we use the version.
 */
{
    [GraphicView setVersion:1];
    DrawPboardType = NXUniqueStringNoCopy(DrawPboardType);
    return self;
}

/* Creation methods. */

static void initClassVars()
/*
 * Sets up any default values.
 */
{
    static BOOL registered = NO;
    const char *validSendTypes[4];
    const char *validReturnTypes[4];

    if (!KeyMotionDeltaDefault) {
	const char *value = NXGetDefaultValue([NXApp appName], "KeyMotionDelta");
	if (value) KeyMotionDeltaDefault = atof(value);
	KeyMotionDeltaDefault = MAX(KeyMotionDeltaDefault, 1.0);
    }
    if (!registered) {
	registered = YES;
	validSendTypes[0] = NXPostScriptPboardType;
	validSendTypes[1] = NXTIFFPboardType;
	validSendTypes[2] = DrawPboardType;
	validSendTypes[3] = NULL;
	[NXApp registerServicesMenuSendTypes:validSendTypes andReturnTypes:[NXImage imagePasteboardTypes]];
	validReturnTypes[0] = DrawPboardType;
	validReturnTypes[1] = NULL;
	[NXApp registerServicesMenuSendTypes:validSendTypes andReturnTypes:validReturnTypes];
    }
}

- initFrame:(const NXRect *)frameRect
/*
 * Initializes the view's instance variables.
 * This view is considered important enough to allocate it a gstate.
 */
{
    [super initFrame:frameRect];
    glist = [[List allocFromZone:[self zone]] init];
    slist = [[List allocFromZone:[self zone]] init];
    cacheWindow = createCache(&bounds.size, [self zone]);
    gvFlags.grid = 10;
    gvFlags.gridDisabled = 1;
    [self allocateGState];
    gridGray = DEFAULT_GRID_GRAY;
    PSInit();
    currentGraphic = [Rectangle class];	      /* default graphic */
    currentGraphic = [self currentGraphic];   /* trick to allow NXApp to control currentGraphic */
    editView = createEditView(self);
    initClassVars();
    
    #ifndef DRAW_FILTER		// OOE 2.0
    [self registerForDragging];
    #endif
    
    return self;
}

/* Free method */

- free
{
    if (gupCoords) {
	NX_FREE(gupCoords);
	NX_FREE(gupOps);
	NX_FREE(gupBBox);
    }
    [glist freeObjects];
    [glist free];
    [slist free];
    [cacheWindow free];
    if (![editView superview]) [editView free];
    return [super free];
}

/* Used by Change's */

- setGroupInSlist:(BOOL)setting
{
    gvFlags.groupInSlist = setting;
    return self;
}

- resetGroupInSlist
{
    gvFlags.groupInSlist = checkForGroup(slist);
    return self;
}

- resetLockedFlag
{
    int i, count;

    gvFlags.locked = NO;
    count = [glist count];
    for (i = 0; (i < count) && (!gvFlags.locked); i++) 
        if ([[glist objectAt:i] isLocked])
	    gvFlags.locked = YES;
    
    return self;
}

- redrawGraphics:graphicsList afterChangeAgent:changeAgent performs:(SEL)aSelector 
{
    NXRect afterBounds, beforeBounds;

    if ([graphicsList count]) {
        [self getBBox:&beforeBounds of:graphicsList];
	[changeAgent perform:aSelector];
        [self getBBox:&afterBounds of:graphicsList];
	NXUnionRect(&beforeBounds, &afterBounds);
	[self cache:&afterBounds];	// recache after change object did something
    } else {
	[changeAgent perform:aSelector];
    }
    
    return self;	
}

/* Public interface methods. */

- (BOOL)isEmpty
{
    return [glist count] == 0;
}

- (BOOL)hasEmptySelection
{
    return [slist count] == 0;
}

- dirty
{
    if ([[window delegate] respondsTo:@selector(dirty:)]) [[window delegate] dirty:self];
    return self;
}

- getSelection
/*
 * Resets slist by going through the glist and locating all the Graphics
 * which respond YES to the isSelected method.
 */
{
    int i;
    Graphic *graphic;

    [slist free];
    slist = [[List allocFromZone:[self zone]] init];
    gvFlags.groupInSlist = NO;
    i = [glist count];
    while (i--) {
	graphic = [glist objectAt:i];
	if ([graphic isSelected]) {
	    [slist insertObject:graphic at:0];
	    gvFlags.groupInSlist = gvFlags.groupInSlist || [graphic isKindOf:[Group class]];
	}
    }

    return self;
}


- graphicsPerform:(SEL)aSelector
/*
 * Performs the given aSelector on each member of the slist, then
 * recaches and redraws the larger of the area covered by the objects before
 * the selector was applied and the area covered by the objects after the
 * selector was applied.  If you want to perform a method on each item
 * in the slist and NOT redraw, then use the List method makeObjectsPerform:.
 */
{
    int i, count;
    Graphic *graphic;
    NXRect eb, affectedBounds;

    count = [slist count];
    if (count) {
	[[slist objectAt:0] getExtendedBounds:&affectedBounds];
	for (i = 1; i < count; i++) {
	    graphic = [slist objectAt:i];
	    NXUnionRect([graphic getExtendedBounds:&eb], &affectedBounds);
	}
	for (i = 0; i < count; i++) {
	    graphic = [slist objectAt:i];
	    [graphic perform:aSelector];
	    NXUnionRect([graphic getExtendedBounds:&eb], &affectedBounds);
	}
	[self cache:&affectedBounds];	// graphicsPerform:
    }	

    return self;
}

- graphicsPerform:(SEL)aSelector with:(void *)argument
{
    int i, count;
    Graphic *graphic;
    NXRect eb, affectedBounds;

    count = [slist count];
    if (count) {
	[[slist objectAt:0] getExtendedBounds:&affectedBounds];
	for (i = 1; i < count; i++) {
	    graphic = [slist objectAt:i];
	    NXUnionRect([graphic getExtendedBounds:&eb], &affectedBounds);
	}
	for (i = 0; i < count; i++) {
	    graphic = [slist objectAt:i];
	    [graphic perform:aSelector with:argument];
	    NXUnionRect([graphic getExtendedBounds:&eb], &affectedBounds);
	}
	[self cache:&affectedBounds];	// graphicsPerform:with:
    }

    return self;
}

- cache:(const NXRect *)rect andUpdateLinks:(BOOL)updateLinks
/*
 * Draws all the Graphics intersected by rect into the off-screen cache,
 * then composites the rect back to the screen (but does NOT flushWindow).
 * If updateLinks is on, then we check to see if the redrawn area intersects
 * an area that someone has created a link to (see gvLinks.m).
 */
{
    gvFlags.cacheing = YES;
    [self drawSelf:rect :1];
    gvFlags.cacheing = NO;

    if ([self canDraw]) {
	[self lockFocus];
	[self drawSelf:rect :1];
	[self unlockFocus];
    }

    #ifndef DRAW_FILTER		// OOE 2.0
    if (updateLinks && !gvFlags.suspendLinkUpdate) [self updateTrackedLinks:rect];
    #endif
    
    return self;
}

- cache:(const NXRect *)rect
{
    return [self cache:rect andUpdateLinks:YES];
}

- cacheAndFlush:(const NXRect *)rect
{
    [self cache:rect];	// cacheAndFlush:
    [window flushWindow];
    return self;
}



#ifndef DRAW_FILTER		// OOE 2.0
- insertGraphic:(Graphic *)graphic
/*
 * Inserts the specified graphic into the glist and draws it.
 * The new graphic will join the selection, not replace it.
 */
{
    NXRect eb;

    if (graphic) {
	if ([graphic isSelected]) [slist insertObject:graphic at:0];
	[glist insertObject:graphic at:0];
	[graphic wasAddedTo:self];
	[self cache:[graphic getExtendedBounds:&eb]];	// insertGraphic:
	if ([graphic isKindOf:[Group class]]) gvFlags.groupInSlist = YES;
	[window flushWindow];
    }

    return self;
}
#endif



#ifndef DRAW_FILTER		// OOE 2.0
- removeGraphic:(Graphic *)graphic
/*
 * Removes the graphic from the GraphicView and redraws.
 */
{
    int i;
    NXRect eb;
    Graphic *g = nil;

    if (!graphic) return self;

    i = [glist count];
    while (g != graphic && i--) g = [glist objectAt:i];
    if (g == graphic) {
	[g getExtendedBounds:&eb];
	[glist removeObjectAt:i];
	[graphic wasRemovedFrom:self];
	[slist removeObject:g];
	if ([g isKindOf:[Group class]]) gvFlags.groupInSlist = checkForGroup(slist);
	[self cache:&eb];	// removeGraphic:
	[window flushWindow];
    }

    return self;
}
#endif



- (Graphic *)selectedGraphic
/*
 * If there is one and only one Graphic selected, this method returns it.
 */
{
    if ([slist count] == 1) {
	Graphic *graphic = [slist objectAt:0];
	return [graphic isKindOf:[Group class]] ? nil : graphic;
    } else {
	return nil;
    }
}

- (List *)selectedGraphics
/*
 * Result valid only immediately after call. GraphicView 
 * reserves the right to free the list without warning.
 */
{
    return slist;
}

- (List *)graphics
/*
 * Result valid only immediately after call. GraphicView 
 * reserves the right to free the list without warning.
 */
{
    return glist;
}


/* Public methods for importing foreign data types into the GraphicView */

#define LOAD_IMAGE NXLocalString("Import Data", NULL, "Title of the alert which lets the user know that the image he has incorporated into his document does not fit.")
#define IMAGE_TOO_LARGE NXLocalString("The imported data is too large to fit on the page.  Resize it to fit?", NULL, "Alert message which asks user if he wants to scale an image or some text that he is incorporating into his document to fit within the size of the document.")
#define SCALE NXLocalString("Resize", NULL, "Button choice which allows the user to scale an image he is incorporating into his document so that it will fit within the size of the document.")
#define DONT_SCALE NXLocalString("Don't Resize", NULL, "Button choice which allows the user to leave an image he is incorporating into his document the same size even though it will not fit within the size of his document.")

#ifndef DRAW_FILTER		// OOE 2.0
- placeGraphic:(Graphic *)graphic at:(const NXPoint *)location
/*
 * Places the graphic centered at the given location on the page.
 * If the graphic is too big, the user is asked whether the graphic
 * should be scaled.
 */
{
    int scale;
    NXPoint offset;
    float sx, sy, factor;
    NXRect gbounds, myBounds, visibleRect;
    id change;

    if (graphic) {
	[graphic getExtendedBounds:&gbounds];
	if (gbounds.size.width > bounds.size.width || gbounds.size.height > bounds.size.height) {
	    scale = NXRunAlertPanel(LOAD_IMAGE, IMAGE_TOO_LARGE, SCALE, DONT_SCALE, CANCEL);
	    if (scale < 0) {
		[graphic free];
		return nil;
	    } else if (scale > 0) {
		sx = (bounds.size.width / gbounds.size.width) * 0.95;
		sy = (bounds.size.height / gbounds.size.height) * 0.95;
		factor = MIN(sx, sy);
		gbounds.size.width *= factor;
		gbounds.size.height *= factor;
		[graphic sizeTo:&gbounds.size];
	    }
	}
	if (location) {
	    [graphic centerAt:location];
	} else {
	    [self getVisibleRect:&visibleRect];
	    visibleRect.origin.x += floor(visibleRect.size.width / 2.0 + 0.5);
	    visibleRect.origin.y += floor(visibleRect.size.height / 2.0 + 0.5);
	    [graphic centerAt:&visibleRect.origin];
	}
	[graphic getExtendedBounds:&gbounds];
	myBounds = bounds;
	NXContainRect(&myBounds, &gbounds);
	offset.x = bounds.origin.x - myBounds.origin.x;
	offset.y = bounds.origin.y - myBounds.origin.y;
	if (offset.x || offset.y) [graphic moveBy:&offset];

	change = [[CreateGraphicsChange alloc] initGraphicView:self graphic:graphic];
	[change startChangeIn:self];
	    [self deselectAll:self];
	    [graphic select];
	    [self insertGraphic:graphic];
	    [self scrollGraphicToVisible:graphic];
	[change endChange];
    }

    return graphic;
}
#endif


/* Methods overridden from superclass. */

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Overrides View's sizeTo:: so that the cacheWindow is resized when
 * the View is resized.
 */
{
    if (width != bounds.size.width || height != bounds.size.height) {
	[super sizeTo:width :height];
	[cacheWindow free];
	cacheWindow = createCache(&bounds.size, [self zone]);
	
	#ifndef DRAW_FILTER		// OOE 2.0    
	[self resetGUP];
	#endif
	
	[self cache:&bounds andUpdateLinks:NO];
    }
    return self;
}



/* Accepting becoming the First Responder */

- (BOOL)acceptsFirstResponder
/*
 * GraphicView always wants to become the first responder when it is
 * clicked on in a window, so it returns YES from this method.
 */
{
    return YES;
}

- (BOOL)acceptsFirstMouse
/*
 * GraphicView really only wants to accept first mouse if that
 * first mouse starts a drag, but there is no way to detect that
 * case, so we're stuck accepting all first mouse events!
 */
{
    return YES;
}

/* Printing */

- beginSetup
/*
 * Spit out the custom PostScript defs.
 */
{
    [super beginSetup];
    PSInit();
    return self;
}

/*
 * These two method set and get the factory object used to create new
 * Graphic objects (i.e. the subclass of Graphic to use).
 * They are kind of weird since they check to see if the
 * Application object knows what the current graphic is.  If it does, then
 * it lets it handle these methods.  Otherwise, it determines the
 * current graphic by querying the sender to find out what its title is
 * and converts that title to the name to a factory object.  This allows
 * the GraphicView to stand on its own, but also use an application wide
 * tool palette if available.
 * If the GraphicView handles the current graphic by itself, it does so
 * by querying the sender of setCurrentGraphic: to find out its title.
 * It assumes, then, that that title is the name of the factory object to
 * use and calls objc_getClass() to get a pointer to it.
 * If the application is not control what our current graphic is, then
 * we restrict creations to be made only when the control key is down.
 * Otherwise, it is the other way around (control key leaves creation
 * mode).  This is due to the fact that the application can be smart
 * enough to set appropriate cursors when a tool is on.  The GraphicView
 * can't be.
 */

- (Graphic *)currentGraphic
{
    if ([NXApp respondsTo:@selector(currentGraphic)]) {
	return [NXApp currentGraphic];
    } else {
	return currentGraphic;
    }
}

- setCurrentGraphic:sender
{
    currentGraphic = objc_getClass([[sender selectedCell] title]);
    return self;
}

/* These methods write out the form information. */

- (BOOL)hasFormEntries
{
    int i;
    for (i = [glist count]-1; i >= 0; i--) {
	if ([[glist objectAt:i] isFormEntry]) return YES;
    }
    return NO;
}

- writeFormEntriesToFile:(const char *)file
{
    int i;
    NXStream *stream;

    stream = NXOpenMemory(NULL, 0, NX_WRITEONLY);
    NXPrintf(stream, "Page Size: w = %d, h = %d\n", (int)bounds.size.width, (int)bounds.size.height);
    for (i = [glist count]-1; i >= 0; i--) {
	[[glist objectAt:i] writeFormEntryToStream:stream];
    }
    NXSaveToFile(stream, file);
    NXCloseMemory(stream, NX_FREEBUFFER);

    return self;
}




/* Archiver-related methods. */

- awake
/*
 * After the GraphicView is unarchived, its cache must be created.
 * If we are loading in this GraphicView just to print it, then we need
 * not load up our cache.
 */
{
    PSInit();
    if (!InMsgPrint) {
	cacheWindow = createCache(&bounds.size, [self zone]);
	[self cache:&bounds andUpdateLinks:NO];
    }
    initClassVars();
    
    #ifndef DRAW_FILTER		// OOE 2.0
    [self registerForDragging];
    #endif
    
    return [super awake];
}

- write:(NXTypedStream *)stream
/*
 * Writes out the glist and the flags.
 * No need to write out the slist since it can be regenerated from the glist.
 * We also ensure that no Text object that might be a subview of the
 * editView gets written out by removing all subviews of the editView.
 */
{
    [super write:stream];
    NXWriteTypes(stream, "@sf", &glist, &gvFlags, &gridGray);
    NXWriteObject(stream, editView);
    return self;
}

- read:(NXTypedStream *)stream
/*
 * Reads in the glist and the flags, and regenerates the slist from the glist.
 */
{
    int i;
    List *evsvs;
    Graphic *graphic, *newGraphic;

    [super read:stream];
    NXReadTypes(stream, "@sf", &glist, &gvFlags, &gridGray);
    for (i = [glist count]-1; i >= 0; i--) {
	graphic = [glist objectAt:i];
	newGraphic = [graphic replaceWithImage];
	if (graphic != newGraphic) {
	    if (graphic) {
		[glist replaceObjectAt:i with:newGraphic];
	    } else {
		[glist removeObjectAt:i];
	    }
	}
    }
    [self getSelection];
	
    #ifndef DRAW_FILTER		// OOE 2.0    
    [self resetGUP];
    #endif

    if (NXTypedStreamClassVersion(stream, [self name]) < 1) {
	editView = createEditView(self);
    } else {
	editView = NXReadObject(stream);
    }

    evsvs = [editView subviews];
    for (i = [evsvs count]-1; i >= 0; i--) [[evsvs objectAt:i] free];

    return self;
}



@end
