#ifndef DRAW_FILTER		// OOE 2.0


#import "draw.h"

#import "GraphicView_C_MenuUpdates.h"

@implementation GraphicView (MenuUpdates)

/* Methods to deal with being/becoming the First Responder */

/* Strings that appear in menus. */

static const char *HIDE_GRID;
static const char *SHOW_GRID;
static const char *TURN_GRID_OFF;
static const char *TURN_GRID_ON;
static const char *SHOW_LINKS;
static const char *HIDE_LINKS;
static BOOL menuStringsInitted = NO;

static void initMenuItemStrings(void)
{
    HIDE_GRID = NXLocalString("Hide Grid", NULL, "Menu item which hides the background grid which the user can overlay on his document.");
    SHOW_GRID = NXLocalString("Show Grid", NULL, "Menu item which shows a background grid which the user can overlay on his document.");
    TURN_GRID_OFF = NXLocalString("Turn Grid Off", NULL, "Menu item which turns off the background grid so that items do not move and resize on even grid boundaries.  It does not hide the grid if it is currently showing.");
    TURN_GRID_ON = NXLocalString("Turn Grid On", NULL, "Menu item which turns a background grid on so that the user's actions are rounded to even grid boundaries.  It does not show the grid if the grid is currently hidden.");
    SHOW_LINKS = NXLocalString("Show Links", NULL, "Menu item which turns on the borders around linked items (using Object Links).");
    HIDE_LINKS = NXLocalString("Hide Links", NULL, "Menu item which turns off the borders around linked items (using Object Links).");
    menuStringsInitted = YES;
}

/* Validates whether a menu command makes sense now */

static BOOL updateMenuItem(MenuCell *menuCell, const char *zeroItem, const char *oneItem, BOOL state)
{
    if (state) {
	if (strcmp([menuCell title], zeroItem) || [menuCell tag] != 0) {
	    [menuCell setTitle:zeroItem];
	    [menuCell setTag:0];
	    [menuCell setEnabled:NO];	// causes it to get redrawn
	}
    } else {
	if (strcmp([menuCell title], oneItem) || [menuCell tag] != 1) {
	    [menuCell setTitle:oneItem];
	    [menuCell setTag:1];
	    [menuCell setEnabled:NO];	// causes it to get redrawn
	}
    }

    return YES;
}

- (BOOL)validateCommand:(MenuCell *)menuCell
/*
 * Can be called to see if the specified action is valid on this view now.
 * It returns NO if the GraphicView knows that action is not valid now,
 * otherwise it returns YES.  Note the use of the Pasteboard change
 * count so that the GraphicView does not have to look into the Pasteboard
 * every time paste: is validated.
 */
{
    Pasteboard *pb;
    int i, count, gcount;
    SEL action = [menuCell action];
    static BOOL pboardHasPasteableType = NO;
    static BOOL pboardHasPasteableLink = NO;
    static int cachedPasteboardChangeCount = -1;
    #ifdef OOE_SERVER
    BOOL clipSel = [self isClipRectSelected];
    #else
    BOOL clipSel = NO;
    #endif
    
    if (!menuStringsInitted) initMenuItemStrings();
    
    if (action == @selector(bringToFront:)) {
    
    	#ifdef OOE_SERVER
    	if(clipSel) return NO;
	#endif
	
	if ((count = [slist count]) && [glist count] > count) {
	    for (i = 0; i < count; i++) {
		if ([slist objectAt:i] != [glist objectAt:i]) {
		    return YES;
		}
	    }
	}
	return NO;
	
    } else if (action == @selector(sendToBack:)) {
    
    	#ifdef OOE_SERVER
    	if(clipSel) return NO;
	#endif
	
	if ((count = [slist count]) && (gcount = [glist count]) > count) {
	    for (i = 1; i <= count; i++) {
		if ([slist objectAt:count-i] != [glist objectAt:gcount-i]) {
		    return YES;
		}
	    }
	}
	return NO;
	
    } else if (action == @selector(group:) ||
	action == @selector(align:)) {
	
    	#ifdef OOE_SERVER
    	if(clipSel) return NO;
	#endif
	
	return([slist count] > 1);
	
    } else if (action == @selector(ungroup:)) {

    
    	#ifdef OOE_SERVER
    	if(clipSel) return NO;
	#endif
	
	return(gvFlags.groupInSlist && [slist count] > 0);
	
    } else if (action == @selector(spellCheck:)) {
    
	for (i = [glist count]-1; i >= 0; i--) 
		if ([[glist objectAt:i] isKindOf:[TextGraphic class]]) 
			return YES;
	return NO;
	
    } else if (action == @selector(deselectAll:) ||
		action == @selector(changeAspectRatio:)) {
	
	return([slist count] > 0);
	
    #ifdef OOE_SERVER
	
    } else if (action == @selector(lock:) ||
	action == @selector(cut:) ||
	action == @selector(copy:)) {
	
	return(!clipSel && [slist count] > 0);
    
    #endif

    } else if (action == @selector(alignToGrid:) ||
	action == @selector(sizeToGrid:)) {
	
	return(GRID > 1 && [slist count] > 0);
    
    } else if (action == @selector(unlock:)) {
	return gvFlags.locked;
    
    } else if (action == @selector(selectAll:)) {
	return([glist count] > [slist count]);
	
    } else if (action == @selector(paste:) || 
    		action == @selector(pasteAndLink:) || 
		action == @selector(link:)) {
	
	#ifdef OOE_SERVER
	if((action == @selector(pasteAndLink:) ||
	    action == @selector(link:)) &&
	    [self clipRect] != nil)
	    	return NO;
	#endif
	
	
	pb = [Pasteboard new];
	count = [pb changeCount];
	
	if (count != cachedPasteboardChangeCount) {
	    cachedPasteboardChangeCount = count;
	    pboardHasPasteableType = (DrawPasteType([pb types]) != NULL);
	    pboardHasPasteableLink = pboardHasPasteableType ? 
	    	IncludesType([pb types], NXDataLinkPboardType) : NO;
   	 }
    
    	return (action == @selector(paste:)) ? 
		pboardHasPasteableType : pboardHasPasteableLink;
		
    } else if (action == @selector(hideGrid:)) {
	return (gvFlags.grid >= 4) ? 
		updateMenuItem(menuCell, HIDE_GRID, SHOW_GRID,
		 [self gridIsVisible]) : NO;
    
    } else if (action == @selector(enableGrid:)) {
	return (gvFlags.grid > 1) ? 
		updateMenuItem(menuCell, TURN_GRID_OFF, 
			TURN_GRID_ON, [self gridIsEnabled]) : NO;
    
    } else if (action == @selector(showLinks:)) {
    
	#ifdef OOE_SERVER
	if([self clipRect] != nil) return NO;
	#endif

	return linkManager ? 
		updateMenuItem(menuCell, 
			HIDE_LINKS, SHOW_LINKS, 
				[linkManager areLinkOutlinesVisible]) : NO;
    
    } else if (action == @selector(print:)) {
	return([glist count] > 0);
    }

    return YES;
}

@end


#endif



