#ifndef DRAW_FILTER		// OOE 2.0


#import "draw.h"

#import "GraphicView_C_MouseEvents.h"

@implementation GraphicView (MouseEvents)

- mouseDown:(NXEvent *)event
/*
 * This method handles a mouse down.
 *
 * If a current tool is in effect, then the mouse down causes a new
 * Graphic to begin being created.  Otherwise, the selection is modified
 * either by adding elements to it or removing elements from it, or moving
 * it.  Here are the rules:
 *
 * Tool in effect
 *    Shift OFF
 *	create a new Graphic which becomes the new selection
 *    Shift ON
 *	create a new Graphic and ADD it to the current selection
 *    Control ON
 *	leave creation mode, and start selection
 * Otherwise
 *    Shift OFF
 *	a. Click on a selected Graphic -> select graphic further back
 *	b. Click on an unselected Graphic -> that Graphic becomes selection
 *    Shift ON
 *	a. Click on a selected Graphic -> remove it from selection
 *	b. Click on unselected Graphic -> add it to selection
 *    Alternate ON
 *	if no affected graphic, causes drag select to select only objects
 *	completely contained within the dragged box.
 *
 * Essentially, everything works as one might expect except the ability to
 * select a Graphic which is deeper in the list (i.e. further toward the
 * back) by clicking on the currently selected Graphic.
 *
 * This is a very hairy mouseDown:.  Most need not be this scary.
 */
{
    NXPoint p;
    NXRect eb;
    int i, corner, oldMask;
    id change, factory;
    Graphic *g = nil, *startg = nil;
    BOOL shift, control, gotHit = NO, deepHit = NO, didDrag = NO;

    /*
     * You only need to do the following line in a mouseDown: method if
     * you receive this message because one of your subviews gets the
     * mouseDown: and does not respond to it (thus, it gets passed up the
     * responder chain to you).  In this case, our editView receives the
     * mouseDown:, but doesn't do anything about it, and when it comes
     * to us, we want to become the first responder.
     *
     * Normally you won't have a subview which doesn't do anything with
     * mouseDown:, in which case, you need only return YES from the
     * method acceptsFirstResponder (see that method below) and will NOT
     * need to do the following makeFirstResponder:.  In other words,
     * don't put the following line in your mouseDown: implementation!
     *
     * Sorry about confusing this issue ... 
     */

    if ([window firstResponder] != self) {
	if (event->data.mouse.click < 2) {
	    [window makeFirstResponder:self];
	} else {
	    return [[window firstResponder] mouseDown:event];
	}
    }

    shift = (event->flags & NX_SHIFTMASK) ? YES : NO;
    control = (event->flags & NX_CONTROLMASK) ? YES : NO;

    p = event->location;
    [self convertPoint:&p fromView:nil];

    oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK|NX_MOUSEUPMASK];

    i = 0;	// See if a Graphic wants to handle this event itself
    [self lockFocus];
    do {
	g = [glist objectAt:i++];
	if ([g handleEvent:event at:&p inView:self]) return self;
    } while (g != nil);
    [self unlockFocus];

    factory = [self currentGraphic];
    if (!control && (factory || (event->data.mouse.click == 2))) {
    
	id editFactory = factory;
	
	if ((event->data.mouse.click == 2) &&
	    ![editFactory isEditable]) 
	    	editFactory = [TextGraphic class];
	
	if ([editFactory isEditable]) {	/* if editable, try to edit one */
	    i = 0;
	    g = [glist objectAt:i++];
	    while (g != nil) {
		if ([g isKindOf:editFactory] && [g hit:&p]) {
		    if ([g isSelected]) {
			[g deselect];
			[self cache:[g getExtendedBounds:&eb] 
					andUpdateLinks:NO];
			[slist removeObject:g];
		    }
		    [g edit:event in:editView];
		    goto done;
		}
		g = [glist objectAt:i++];
	    }
	}
    }
    
    if (!control && factory) {
	if (factory && !g) {	/* not editing or no editable graphic found */
	    g = [[factory allocFromZone:[self zone]] init];
	    if ([NXApp respondsTo:@selector(inspectorPanel)]) {
		[[[NXApp inspectorPanel] delegate] initializeGraphic:g];
	    }
	    if ([g create:event in:self]) {
		change = [[CreateGraphicsChange alloc] 
				initGraphicView:self graphic:g];
		[change startChange];
		    if (!shift) [self deselectAll:self];
		    [self insertGraphic:g];
		    [g edit:NULL in:editView];
		[change endChange];
	    } else {
		[g free];
	    }
	}
    } else {		/* selecting/resizing/moving */
	i = 0;
	g = [glist objectAt:i++];
	while (g != nil && !gotHit) {
	    corner = [g knobHit:&p];
	    if (corner > 0) {			/* corner hit */
		gotHit = YES;
		change = [[ResizeGraphicsChange alloc] 
				initGraphicView:self graphic:g];
		[change startChange];
		    [g resize:event by:corner in:self];
		[change endChange];
	    } else if (corner) {		/* complete miss */
		g = [glist objectAt:i++];
	    } else g = nil;			/* non-corner opaque hit */
	}
	i = 0;
	if (!gotHit) g = [glist objectAt:i++];
	while (g && !gotHit && !deepHit) {
	    if ([g isSelected] && [g hit:&p]) {
		if (shift) {
		    gotHit = YES;
		    [g deselect];
		    [self cache:[g getExtendedBounds:&eb] andUpdateLinks:NO];
		    [slist removeObject:g];
		    if ([g isKindOf:[Group class]]) 
		    		gvFlags.groupInSlist = checkForGroup(slist);
		} else {
		    gotHit = [self move:event];
		    if (!gotHit) {
			deepHit = ![g isOpaque];
			if (!deepHit) gotHit = YES;
		    } 
		}
	    }
	    g = [glist objectAt:i++];
	}
	startg = g;
	if (!gotHit) do {
	    if (!g) {
		i = 0;
		g = [glist objectAt:i++];
	    }
	    
	    
	    if(![g isSelected] && [g hit:&p]) {
	    
		//#ifdef OOE_SERVER
	    	//if(g == [self clipRect] && ![g borderHit:&p])
		//   	break;
		//#endif
		
		gotHit = YES;
		
		if (!shift) {
		    [self deselectAll:self];
		    [slist addObject:g];
		    gvFlags.groupInSlist = [g isKindOf:[Group class]];
		}
		
		[g select];
		
		if (shift) [self getSelection];
		
		if (deepHit || ![self move:event]) {
		
		    [self cache:[g getExtendedBounds:&eb] andUpdateLinks:NO];
		}
		
	    } else {
	    
		g = [glist objectAt:i++];
	    }
	    
	} while (!gotHit && g != startg);

	if (!gotHit && !deepHit) {
	    if (!shift) {
		[self lockFocus];
		[self deselectAll:self];
		[self unlockFocus];
		didDrag = YES;
	    }
	    [self dragSelect:event];
	}
    }
done:
    if (!didDrag && dragRect) {
	NX_FREE(dragRect);
	dragRect = NULL;
    }
    gvFlags.selectAll = NO;

    [window flushWindow];
    [window setEventMask:oldMask];

    return self;
}

@end


#endif



