//
//	Xanthus
//	Copyright (c) 1992-95 Bjorn Backlund.  
//	All rights reserved. 
//

#import "draw.h"

#import "GraphicView_C_Pasteboard_Copy.h"

@implementation GraphicView (Pasteboard_Copy)



/* Lazy Pasteboard evaluation handler */

/*
 * IMPORTANT: The pasteboard:provideData: method is a factory method since the
 * factory object is persistent and there is no guarantee that the INSTANCE of
 * GraphicView that put the Draw format into the Pasteboard will be around
 * to lazily put PostScript or TIFF in there, so we keep one around (actually
 * we only create it when we need it) to do the conversion (scrapper).
 *
 * If you find this part of the code confusing, then you need not even
 * use the provideData: mechanism--simply put the data for all the different
 * types your program knows how to put in the Pasteboard in at the time
 * that you declareTypes:.
 */

/*
 * Converts the data in the Pasteboard from Draw internal format to
 * either PostScript or TIFF using the writeTIFFToStream: and writePSToStream:
 * methods.  It sends these messages to the scrapper (a GraphicView cached
 * to perform this very function).  Note that the scrapper view is put in
 * a window, but that window is off-screen, has no backing store, and no
 * title (and is thus very cheap).
 */

#ifndef DRAW_FILTER		// OOE 2.0
+ convert:(NXTypedStream *)ts 
	to:(const char *)type 
	using:(SEL)writer 
	toPasteboard:(Pasteboard *)pb
{
    Window *w;
    List *list;
    NXZone *zone;
    NXStream *stream;
    GraphicView *scrapper;
    NXRect scrapperFrame = {{0.0, 0.0}, {8.5*72.0, 11.0*72.0}}; // OOE 2
    NXRect graphicBBox;
    
    if (!ts) return self;

    zone = NXCreateZone(vm_page_size, vm_page_size, NO);
    NXNameZone(zone, "Scrapper");
    scrapper = [[GraphicView allocFromZone:zone] initFrame:&scrapperFrame];
    NXSetTypedStreamZone(ts, zone);
    list = NXReadObject(ts);
    
    /* ORIG
    [scrapper getBBox:&scrapperFrame of:list];
   
    scrapperFrame.size.width += scrapperFrame.origin.x;
    scrapperFrame.size.height += scrapperFrame.origin.y;
    scrapperFrame.origin.x = scrapperFrame.origin.y = 0.0;
    */
    
    [scrapper getBBox:&graphicBBox of:list]; 	// OOE 2
    NXUnionRect(&graphicBBox, &scrapperFrame);	// OOE 2
    
    [scrapper sizeTo:scrapperFrame.size.width :scrapperFrame.size.height];
    
    w = [[Window allocFromZone:zone] initContent:&scrapperFrame
					   style:NX_PLAINSTYLE
					 backing:NX_NONRETAINED
				      buttonMask:0
					   defer:NO];
    [w reenableDisplay];
    [w setContentView:scrapper];
    stream = NXOpenMemory(NULL, 0, NX_WRITEONLY);
    [scrapper perform:writer with:(id)stream with:list];
    [pb writeType:type fromStream:stream];
    NXCloseMemory(stream, NX_FREEBUFFER);
    [list freeObjects];
    [list free];
    [w free];
    NXDestroyZone(zone);

    return self;
}
#endif




/*
 * Called by the Pasteboard whenever PostScript or TIFF data is requested
 * from the Pasteboard by some other application.  The current contents of
 * the Pasteboard (which is in the Draw internal format) is 
 * taken out and loaded
 * into a stream, then convert:to:using:toPasteboard: is called.  This
 * returns self if successful, nil otherwise.
 */

#ifndef DRAW_FILTER		// OOE 2.0
+ pasteboard:(Pasteboard *)sender provideData:(const char *)type
{
    id retval = nil;
    NXStream *stream;
    NXTypedStream *ts;

    if(type == OOE_ApplicationPboardType) {		// OOE 2.0
    
    	[self addAppInfoToPboard: sender];
    	return self;
    }
    
    if((type == NXPostScriptPboardType) || 
       (type == NXTIFFPboardType)  || 		// OOE 2.0
       (type == OOE_PboardType)) {		// OOE 2.0
    
	if(stream = [sender readTypeToStream:DrawPboardType]) {
	   
	    if(ts = NXOpenTypedStream(stream, NX_READONLY)) {
	    
		retval = self;
		
		if (type == NXPostScriptPboardType) {
		
		    [self convert:ts 
		    	to:type 
			using:@selector(writePSToStream:usingList:) 
			toPasteboard:sender];
			
		} else if (type == NXTIFFPboardType) {
		
		    [self convert:ts 
		    	to:type 
			using:@selector(writeTIFFToStream:usingList:) 
			toPasteboard:sender];
		
		} else if (type == OOE_PboardType) {	// OOE 2.0
		
		    [self convert:ts 
		    	to:type 
			using:@selector(writeOOEToStream:usingList:) 
			toPasteboard:sender];
		
		} else {
		    retval = nil;
		}
		
		NXCloseTypedStream(ts);
	    }
	    NXCloseMemory(stream, NX_FREEBUFFER);
	}
    }

    return retval;
}
#endif


/* Writing data in different forms (other than the internal Draw format) */

/*
 * Writes out the PostScript generated by drawing all the objects in the
 * glist.  The bounding box of the generated encapsulated PostScript will
 * be equal to the bounding box of the objects in the glist (NOT the
 * bounds of the view).
 */

#ifndef DRAW_FILTER		// OOE 2.0
- writePSToStream:(NXStream *)stream
{
    NXRect bbox;

    if (stream) {
	if (([glist count] == 1) && [[glist objectAt:0] canEmitEPS]) {
	    [[glist objectAt:0] writeEPSToStream:stream];
	} else {
	    [self getBBox:&bbox of:glist];
	    [self copyPSCodeInside:&bbox to:stream];
	}
    }

    return self;
}
#endif

/*
 * This is the same as writePSToStream:, but it lets you specify the list
 * of Graphics you want to generate PostScript for (does its job by swapping
 * the glist for the list you provide temporarily).
 */

#ifndef DRAW_FILTER		// OOE 2.0
- writePSToStream:(NXStream *)stream usingList:list
{
    List *savedglist;

    savedglist = glist;
    glist = list;
    [self writePSToStream:stream];
    glist = savedglist;

    return self;
}
#endif

/*
 * Images all of the objects in the glist and writes out the result in
 * the Tagged Image File Format (TIFF).  The image will not have alpha in it.
 */

#ifndef DRAW_FILTER		// OOE 2.0
- writeTIFFToStream:(NXStream *)stream
{
    NXRect sbounds;
    Window *tiffCache;
    NXBitmapImageRep *bm;

    if (!stream) return self;

    if (([glist count] == 1) && [[glist objectAt:0] canEmitTIFF]) {
	[[glist objectAt:0] writeTIFFToStream:stream];
    } else {
	tiffCache = [self createCacheWindow:nil];
	[tiffCache setDepthLimit:NX_TwentyFourBitRGBDepth];
	[self cacheList:glist into:tiffCache withTransparentBackground:NO];
	[self getBBox:&sbounds of:glist];
	[[tiffCache contentView] lockFocus];
	sbounds.origin.x = sbounds.origin.y = 0.0;
	bm = [[NXBitmapImageRep alloc] initData:NULL fromRect:&sbounds];
	[[tiffCache contentView] unlockFocus];
	[bm writeTIFF:stream usingCompression:NX_TIFF_COMPRESSION_LZW];
	[bm free];
	[tiffCache free];
    }

    return self;
}
#endif




/*
 * This is the same as writeTIFFToStream:, but it lets you specify the list
 * of Graphics you want to generate TIFF for (does its job by swapping
 * the glist for the list you provide temporarily).
 */

#ifndef DRAW_FILTER		// OOE 2.0
- writeTIFFToStream:(NXStream *)stream usingList:list
{
    List *savedglist;

    savedglist = glist;
    glist = list;
    [self writeTIFFToStream:stream];
    glist = savedglist;

    return self;
}
#endif









/* Writing the selection to a stream */

#ifndef DRAW_FILTER		// OOE 2.0
- copySelectionAsPSToStream:(NXStream *)stream
{
    return (stream && [slist count]) ? [self writePSToStream:stream usingList:slist] : nil;
}
#endif




#ifndef DRAW_FILTER		// OOE 2.0
- copySelectionAsTIFFToStream:(NXStream *)stream
{
    return (stream && [slist count]) ? [self writeTIFFToStream:stream usingList:slist] : nil;
}
#endif




#ifndef DRAW_FILTER		// OOE 2.0
- copySelectionToStream:(NXStream *)stream
{
    NXTypedStream *ts;

    if ([slist count]) {
	ts = NXOpenTypedStream(stream, NX_WRITEONLY);
	NXWriteRootObject(ts, slist);
	NXCloseTypedStream(ts);
    } else {
	return nil;
    }

    return self;
}
#endif










/* Methods to write to/read from the pasteboard */

/*
 * Puts all the objects in the slist into the Pasteboard by archiving
 * the slist itself.  Also registers the PostScript and TIFF types since
 * the GraphicView knows how to convert its internal type to PostScript
 * or TIFF via the write{PS,TIFF}ToStream: methods.
 */

#ifndef DRAW_FILTER		// OOE 2.0
- copyToPasteboard:(Pasteboard *)pboard types:(NXAtom *)typesList
{
    char *data;
    NXStream *stream;
    const char *types[16];
    int i = 0, length, maxlen;

    if ([slist count]) {
	
	types[i++] = DrawPboardType;
	
	if (!typesList || IncludesType(typesList, NXPostScriptPboardType)) {
	    types[i++] = NXPostScriptPboardType;
	}
	
	if (!typesList || IncludesType(typesList, NXTIFFPboardType)) {
	    types[i++] = NXTIFFPboardType;
	}
	
	// OOE 2.0
	// Add the OOE app info pboard type 
	
	if (!typesList || IncludesType(typesList, OOE_ApplicationPboardType)) {
	    types[i++] = OOE_ApplicationPboardType;
	}
	
	if (!typesList || IncludesType(typesList, OOE_PboardType)) {
	    types[i++] = OOE_PboardType;
	}
	
	// end OOE 2.0
	
	[pboard declareTypes:types 
		num:i 
		owner:[self class]];
	stream = NXOpenMemory(NULL, 0, NX_WRITEONLY);
	
	[self copySelectionToStream:stream];
	NXGetMemoryBuffer(stream, &data, &length, &maxlen);
	
	[pboard writeType:DrawPboardType 
		data:data 
		length:length];
	NXCloseMemory(stream, NX_FREEBUFFER);
	
	[self writeLinkToPasteboard:pboard 
		types:typesList];
	return self;
    
    } else {
	return nil;
    }
}
#endif



#ifndef DRAW_FILTER		// OOE 2.0
- copyToPasteboard:(Pasteboard *)pboard
{
    return [self copyToPasteboard:pboard types:NULL];
}
#endif



@end


