//
//	Xanthus
//	Copyright (c) 1992-95 Bjorn Backlund.  
//	All rights reserved. 
//

#import "draw.h"

#import "GraphicView_C_Pasteboard_Paste.h"

@implementation GraphicView (Pasteboard_Paste)




/*
 * Pastes any data that comes from another application.
 * Basically this is the "else" in pasteFromPasteboard: below if there
 * is no Draw internal format in the Pasteboard.  This is also called
 * from the drag stuff (see gvDrag.m).
 */

#ifndef DRAW_FILTER		// OOE 2.0
- (BOOL)pasteForeignDataFromPasteboard:(Pasteboard *)pboard 
	andLink:(LinkType)doLink 
	at:(const NXPoint *)center
{
    NXDataLink *link = nil;
    Graphic *graphic = nil;

    if (!linkManager) doLink = DontLink;

    if (doLink) link = [[NXDataLink alloc] initFromPasteboard:pboard];
    if (link && (doLink == LinkOnly)) {
	graphic = [[Image allocFromZone:[self zone]] initWithLinkButton];
    } else {
	graphic = [[TextGraphic allocFromZone:[self zone]] 
				initFromPasteboard:pboard];
	if (!graphic) graphic = [[Image allocFromZone:[self zone]] 
					initFromPasteboard:pboard];
    }
    
    [self deselectAll:self];
    
    if (doLink && link) {
	if ([self addLink:link 
		toGraphic:graphic 
		at:center 
		update:UPDATE_NORMALLY]) return YES;
    } else if (graphic) {
	if ([self placeGraphic:graphic at:center]) return YES;
    }

    return NO;
}
#endif





/*
 * Pastes any type available from the specified Pasteboard into the GraphicView.
 * If the type in the Pasteboard is the internal type, then the objects
 * are simply added to the slist and glist.  If it is PostScript or TIFF,
 * then an Image object is created using the contents of
 * the Pasteboard.  Returns a list of the pasted objects (which should be freed
 * by the caller).
 */

#ifndef DRAW_FILTER		// OOE 2.0
- pasteFromPasteboard:(Pasteboard *)pboard 
	andLink:(LinkType)doLink 
	at:(const NXPoint *)center
{
    int i;
    id change;
    NXStream *stream;
    NXTypedStream *ts;
    List *pblist = nil;
    Graphic *graphic = nil;
    BOOL pasteDrawType = NO;

    if (!linkManager) doLink = DontLink;
    if (!doLink) pasteDrawType = IncludesType([pboard types], DrawPboardType);

    if (pasteDrawType) {
	stream = [pboard readTypeToStream:DrawPboardType];
	ts = NXOpenTypedStream(stream, NX_READONLY);
	pblist = NXReadObject(ts);
	if (i = [pblist count]) {
	    change = [[PasteGraphicsChange alloc] 
	    			initGraphicView:self graphics:pblist];
	    [change startChange];
		
		[self deselectAll:self];
		while (i--) {
		
		    graphic = [pblist objectAt:i];
		    [slist insertObject:graphic at:0];
		    [glist insertObject:graphic at:0];
		    if ([graphic mightBeLinked]) {
		    
			BOOL useNewId = ([pboard changeCount] != 
				lastCutChangeCount) || consecutivePastes;
			[graphic readLinkFromPasteboard:pboard 
				usingManager:linkManager 
				useNewIdentifier:useNewId];
		    }
		    gvFlags.groupInSlist = gvFlags.groupInSlist || 
		    			[graphic isKindOf:[Group class]];
		}
	    [change endChange];
	} else {
	    [pblist free];
	    pblist = nil;
	}
	NXCloseTypedStream(ts);
	NXCloseMemory(stream, NX_FREEBUFFER);
   
    } else {
	[self pasteForeignDataFromPasteboard:pboard andLink:doLink at:center];
    }

    return pblist;
}
#endif




/*
 * Pastes from the normal pasteboard.
 * This paste implements "smart paste" which goes like this: if the user
 * pastes in a single item (a Group is considered a single item), then
 * pastes that item again and moves that second item somewhere, then
 * subsequent pastes will be positioned at the same offset between the
 * first and second pastes (this is also known as "transform again").
 */

#ifndef DRAW_FILTER		// OOE 2.0
- paste:sender andLink:(LinkType)doLink
{
    List *pblist;
    NXPoint offset;
    Graphic *graphic;
    Pasteboard *pboard;
    NXRect originalBounds, secondBounds;
    static Graphic *secondPaste;
    static NXPoint pasteOffset;

    pboard = [Pasteboard new];
    pblist = [self pasteFromPasteboard:pboard andLink:doLink at:NULL];

    if (pblist && IncludesType([pboard types], DrawPboardType)) {
    
	graphic = ([pblist count] == 1) ? [pblist objectAt:0] : nil;
	
	if (lastPastedChangeCount != [pboard changeCount]) {
	    
	    consecutivePastes = 0;
	    lastPastedChangeCount = [pboard changeCount];
	    originalPaste = graphic;
	
	} else {
	
	    if (consecutivePastes == 1) {   /* smart paste */
		if (gvFlags.gridDisabled) { /* offset to grid if turned off */
		    pasteOffset.x = 10.0;
		    pasteOffset.y = -10.0;
		} else {
		    pasteOffset.x = (float)gvFlags.grid;
		    pasteOffset.y = -(float)gvFlags.grid;
		}
		secondPaste = graphic;
	    } else if ((consecutivePastes == 2) && graphic) {
		[originalPaste getBounds:&originalBounds];
		[secondPaste getBounds:&secondBounds];
		pasteOffset.x = secondBounds.origin.x - 
					originalBounds.origin.x;
		pasteOffset.y = secondBounds.origin.y - 
					originalBounds.origin.y;
	    }
	    offset.x = pasteOffset.x * consecutivePastes;
	    offset.y = pasteOffset.y * consecutivePastes;
	    [slist makeObjectsPerform:@selector(moveBy:) with:(id)&offset];
	}
	
	consecutivePastes++;
	[self recacheSelection];
    }

    [pblist free];

    return self;
}
#endif

@end


