
#import "draw.h"

#import "GraphicView_C_SelectionPrivate.h"

@implementation GraphicView (SelectionPrivate)

- getBBox:(NXRect *)bbox of:(List *)list
{
    return [self getBBox:bbox of:list extended:YES];
}


/* Private selection management methods. */

- createCacheWindow:(Window *)selectioncache
/*
 * Shares an off-screen window used to draw the selection in so that it
 * can be dragged around.  If the current off-screen window is equal in
 * size or larger than the passed size, then it is simply returned.
 * Otherwise, it is resized to be the specified size.
 */
{
    NXRect rect;

    if (!selectioncache) {
	rect = bounds;
	selectioncache = [Window newContent:&rect
				      style:NX_PLAINSTYLE
				    backing:NX_RETAINED
				 buttonMask:0
				      defer:NO];
	[selectioncache reenableDisplay];
    } else {
	[selectioncache getFrame:&rect];
	if (rect.size.width < bounds.size.width || rect.size.height < bounds.size.height) {
	    [selectioncache sizeWindow:MAX(rect.size.width, bounds.size.width)
				      :MAX(rect.size.height, bounds.size.height)];
	}
    }

    return selectioncache;
}

- selectionCache
{
    static Window *selectioncache = nil;
    return selectioncache = [self createCacheWindow:selectioncache];
}

- getBBox:(NXRect *)bbox of:(List *)list extended:(BOOL)extended
/*
 * Returns a rectangle which encloses all the objects in the list.
 */
{
    int i;
    NXRect eb;

    i = [list count];
    if (i) {
	if (extended) {
	    [[list objectAt:--i] getExtendedBounds:bbox];
	    while (i--) NXUnionRect([[list objectAt:i] getExtendedBounds:&eb], bbox);
	} else {
	    [[list objectAt:--i] getBounds:bbox];
	    while (i--) {
		[[list objectAt:i] getBounds:&eb];
		NXUnionRect(&eb, bbox);
	    }
	}
    } else {
	bbox->size.width = bbox->size.height = 0.0;
    }

    return self;
}

- recacheSelection:(BOOL)updateLinks
 /*
  * Redraws the selection in the off-screen cache (not the selection cache),
  * then composites it back on to the screen.
  */
{
    NXRect sbounds;
    
    if ([slist count]) {
	[self getBBox:&sbounds of:slist];
	gvFlags.cacheing = YES;
	[self drawSelf:&sbounds :1];
	gvFlags.cacheing = NO;
	[self display:&sbounds :1];
	if (updateLinks && !gvFlags.suspendLinkUpdate) [self updateTrackedLinks:&sbounds];
    }

    return self;
}

- recacheSelection
{
    return [self recacheSelection:YES];
}

- compositeSelection:(const NXRect *)sbounds from:(int)gstate
/*
 * Composites from the specified gstate whatever part of sbounds is
 * currently visible in the View.
 */
{
    PScomposite(0.0, 0.0, NX_WIDTH(sbounds), NX_HEIGHT(sbounds),
                gstate, NX_X(sbounds), NX_Y(sbounds), NX_SOVER);
    [window flushWindow];
    NXPing();
    return self;
}

- (int)cacheList:(List *)aList into:(Window *)selectionCache withTransparentBackground:(BOOL)transparentBackground
 /*
  * Caches the selection into the application-wide selection cache
  * window (a window which has alpha in it).  See also: selectionCache:.
  * It draws the objects without their knobbies in the selection cache,
  * but it leaves them selected.  Returns the gstate of the selection
  * cache.
  */
{
    int i;
    NXRect sbounds, scframe;

    [self getBBox:&sbounds of:aList];
    [selectionCache getFrame:&scframe];
    if (scframe.size.width < sbounds.size.width || scframe.size.height < sbounds.size.height) {
	[selectionCache sizeWindow:MAX(scframe.size.width, sbounds.size.width)
				  :MAX(scframe.size.height, sbounds.size.height)];
    }
    [[selectionCache contentView] lockFocus];
    PSsetgray(NX_WHITE);
    PSsetalpha(transparentBackground ? 0.0 : 1.0);	/* 0.0 means fully transparent */
    PStranslate(- sbounds.origin.x, - sbounds.origin.y);
    sbounds.size.width += 1.0;
    sbounds.size.height += 1.0;
    NXRectFill(&sbounds);
    sbounds.size.width -= 1.0;
    sbounds.size.height -= 1.0;
    PSsetalpha(1.0);					/* set back to fully opaque */
    i = [aList count];
    while (i--) [[[[aList objectAt:i] deselect] draw:NULL] select];
    [Graphic showFastKnobFills];
    PStranslate(sbounds.origin.x, sbounds.origin.y);
    [[selectionCache contentView] unlockFocus];

    return [selectionCache gState];
}

- (int)cacheList:(List *)aList into:(Window *)selectionCache
{
    return [self cacheList:aList into:selectionCache withTransparentBackground:YES];
}

- (int)cacheSelection
{
    return [self cacheList:slist into:[self selectionCache] withTransparentBackground:YES];
}

/* Other private methods. */

- cacheGraphic:(Graphic *)graphic
 /*
  * Draws the graphic into the off-screen cache, then composites
  * it back to the screen.
  * NOTE: This ONLY works if the graphic is on top of the list!
  * That is why it is a private method ...
  */
{
    NXRect eb;

    [[cacheWindow contentView] lockFocus];
    [graphic draw:NULL];
    [Graphic showFastKnobFills];
    [[cacheWindow contentView] unlockFocus];
    [self display:[graphic getExtendedBounds:&eb] :1];

    return self;
}


#ifndef DRAW_FILTER		// OOE 2.0
- resetGUP
/*
 * The "GUP" is the Grid User Path.  It is a user path which draws a grid
 * the size of the bounds of the GraphicView.  This gets called whenever
 * the View is resized or the grid spacing is changed.  It sets up all
 * the arguments to DPSDoUserPath() called in drawSelf::.
 */
{
    int x, y, i, j;
    short w, h;

    if (gvFlags.grid < 4) return self;

    x = (int)bounds.size.width / (gvFlags.grid ? gvFlags.grid : 1);
    y = (int)bounds.size.height / (gvFlags.grid ? gvFlags.grid : 1);
    gupLength = (x << 2) + (y << 2);
    if (gupCoords) {
	NX_FREE(gupCoords);
	NX_FREE(gupOps);
	NX_FREE(gupBBox);
    }
    NX_ZONEMALLOC([self zone], gupCoords, short, gupLength);
    NX_ZONEMALLOC([self zone], gupOps, char, gupLength >> 1);
    NX_ZONEMALLOC([self zone], gupBBox, short, 4);
    w = bounds.size.width;
    h = bounds.size.height;
    j = 0;
    for (i = 1; i <= y; i++) {
	gupCoords[j++] = 0.0;
	gupCoords[j++] = i * (gvFlags.grid ? gvFlags.grid : 1);
	gupCoords[j++] = w;
	gupCoords[j] = gupCoords[j-2];
	j++;
    }
    for (i = 1; i <= x; i++) {
	gupCoords[j++] = i * (gvFlags.grid ? gvFlags.grid : 1);
	gupCoords[j++] = 0.0;
	gupCoords[j] = gupCoords[j-2];
	j++;
	gupCoords[j++] = h;
    }
    i = gupLength >> 1;
    while (i) {
	gupOps[--i] = dps_lineto;
	gupOps[--i] = dps_moveto;
    }
    gupBBox[0] = gupBBox[1] = 0;
    gupBBox[2] = bounds.size.width + 1;
    gupBBox[3] = bounds.size.height + 1;

    return self;
}
#endif


@end


